@@ -19,10 +19,20 @@
 from .tika import detectType1, callServer, ServerEndpoint
 
 def from_file(filename):
+    '''
+    Detects MIME type of specified file
+    :param filename: file whose type needs to be detected
+    :return: MIME type
+    '''
     jsonOutput = detectType1('type', filename)
     return jsonOutput[1]
 
 def from_buffer(string):
+    '''
+    Detects MIME type of the buffered content
+    :param string: buffered content whose type needs to be detected
+    :return:
+    '''
     status, response = callServer('put', ServerEndpoint, '/detect/stream', string,
                                   {'Accept': 'text/plain'}, False)
     return response
