@@ -19,10 +19,20 @@
 from .tika import detectLang1, callServer, ServerEndpoint
 
 def from_file(filename):
+    '''
+    Detects language of the file
+    :param filename: path to file whose language needs to be detected
+    :return:
+    '''
     jsonOutput = detectLang1('file', filename)
     return jsonOutput[1]
 
 def from_buffer(string):
+    '''
+    Detects language of content in the buffer
+    :param string: buffered data
+    :return:
+    '''
     status, response = callServer('put', ServerEndpoint, '/language/string', string,
                                   {'Accept': 'text/plain'}, False)
     return response
